SystemOrganization addCategory: #'WebClient-Tests'!

TestCase subclass: #WebClientServerTest
	instanceVariableNames: 'server user password oldAuthHandler oldProxyHandler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WebClient-Tests'!

!WebClientServerTest commentStamp: 'ar 2/24/2010 00:13' prior: 0!
Tests for both WebClient and WebServer.!

!WebClientServerTest methodsFor: 'setup' stamp: 'ar 2/23/2010 23:12'!
port

	^7766! !

!WebClientServerTest methodsFor: 'setup' stamp: 'ar 3/27/2010 01:49'!
setUp
	oldAuthHandler := WebClient defaultAuthCredentials.
	oldProxyHandler := WebClient defaultProxyCredentials.
	WebClient defaultAuthCredentials:[:client :realm :fail| 
		client username = user ifTrue:[fail value].
		client username: user.
		client password: password.
	].
	WebClient defaultProxyCredentials:[:client :realm :fail| 
		client username = user ifTrue:[fail value].
		client username: user.
		client password: password.
	].
	server := WebServer new listenOn: self port.
	server passwordAt: 'user' realm: 'test' put: 'pass'.
! !

!WebClientServerTest methodsFor: 'setup' stamp: 'ar 3/27/2010 01:42'!
tearDown
	WebClient defaultAuthCredentials: oldAuthHandler.
	WebClient defaultProxyCredentials: oldProxyHandler.
	server ifNotNil:[server destroy].
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 5/4/2010 15:32'!
testAuthCredentialsBlock
	"Test client and server handling of digest auth"

	| resp reqHeader usedCustom usedDefault client |
	server addService: '/test/auth' action:[:req | 
		server authenticate: req realm: 'test' methods: #(digest) do:[
			reqHeader := req headerAt: 'Authorization'.
			req send200Response: 'ok'
		].
	].

	usedCustom := usedDefault := false.
	WebClient defaultAuthCredentials:[:wc :realm :fail| 
		usedDefault := true.
		fail value].

	client := WebClient new.
	user := password := nil.
	resp := client httpGet: 'http://localhost:', self port, '/test/auth'.
	self assert: resp code = 401.
	self assert: usedDefault.
	self deny: usedCustom.

	usedCustom := usedDefault := false.
	client authCredentialsBlock:[:wc :realm :fail|
		usedCustom := true.
		client username: 'user'.
		client password: 'pass'.
	].
	resp := client httpGet: 'http://localhost:', self port, '/test/auth'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (reqHeader beginsWith: 'Digest ').
	self assert: usedCustom.
	self deny: usedDefault.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 16:03'!
testAuthRedirectSession
	"Run a test for a full auth-redirect-cookie loop"

	| loginOK finalUrl finalFields firstRedirect client resp |
	server addService: '/login' action:[:req | 
		server authenticate: req realm: 'test' methods: #(digest) do:[ | id |
			loginOK := true.
			server sessionAt: (id := UUID new hex) put: ''.
			req send302Response: (req fields at: 'url' ifAbsent:['/']) 
				do:[:reply| reply setCookie: 'session' value: id path: '/']]].
	server addService: '/action' action:[:req | 
		(server sessionAt: (req cookieAt: 'session')) ifNil:[
			firstRedirect := true.
			req send302Response: '/login?url=', req rawUrl encodeForHTTP.
		] ifNotNil:[
			finalUrl := req url.
			finalFields := req fields.
			req send200Response: 'ok'.
		].
	].
	firstRedirect := loginOK := false.
	server passwordAt: 'squeak' realm: 'test' put: 'foo'.
	client := WebClient new.
	client username: 'squeak'; password: 'foo'.
	resp := client httpGet: 'http://localhost:', self port, '/action/foo/bar?string=hello&number=42'.

	self assert: resp code = 200.
	self assert: firstRedirect.
	self assert: loginOK.
	self assert: finalUrl = '/action/foo/bar'.
	self assert: finalFields size = 2.
	self assert: (finalFields at: 'string') = 'hello'.
	self assert: (finalFields at: 'number') = '42'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 19:36'!
testBasicAuth
	"Test client and server basic auth"

	| resp reqHeader |
	server addService: '/test/auth' action:[:req | 
		server authenticate: req realm: 'test' methods: #(basic) do:[
			reqHeader := req headerAt: 'Authorization'.
			req send200Response: 'ok'
		].
	].
	user := password := nil.
	resp := WebClient httpGet: 'http://localhost::', self port, '/test/auth'.
	self assert: resp code = 401.

	user := 'squeak'. password := 'squeak'.
	resp := WebClient httpGet: 'http://localhost::', self port, '/test/auth'.
	self assert: resp code = 401.

	user := 'user'. password := 'pass'.
	resp := WebClient httpGet: 'http://localhost::', self port, '/test/auth'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (reqHeader beginsWith: 'Basic ').
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 22:51'!
testCookies
	"Test client and server handling of cookies"

	| resp client |
	"Sends the testcookie back if it's set"
	server addService: '/' action:[:req | 
		req send200Response: (req cookieAt: 'testcookie').
	].

	"Sets the cookie"
	server addService: '/setcookie' action:[:req | 
		req send200Response: 'ok' contentType: 'text/plain' 
			do:[:reply| reply setCookie: 'testcookie' value: '123' path: '/cookie']].

	client := WebClient new.
	resp := client httpGet: 'http://localhost:', self port, '/setcookie'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (resp setCookieAt: 'testcookie') value = '123'.

	resp := client httpGet: 'http://localhost:', self port, '/cookie/test'.
	self assert: resp code = 200.
	self assert: resp content = '123'.

	resp := client httpGet: 'http://localhost:', self port, '/nocookie/test'.
	self assert: resp code = 200.
	self assert: resp content = ''.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/23/2010 23:13'!
testDefault404
	"Test that a new server responds with 404 to anything"

	| resp |
	resp := WebClient httpGet: 'http://localhost::', self port, '/'.
	self assert: resp code = 404.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 3/27/2010 01:51'!
testDigestAuth
	"Test client and server handling of digest auth"

	| resp reqHeader |
	server addService: '/test/auth' action:[:req | 
		server authenticate: req realm: 'test' methods: #(digest) do:[
			reqHeader := req headerAt: 'Authorization'.
			req send200Response: 'ok'
		].
	].

	user := password := nil.
	resp := WebClient httpGet: 'http://localhost:', self port, '/test/auth'.
	self assert: resp code = 401.

	user := 'squeak'. password := 'squeak'.
	resp := WebClient httpGet: 'http://localhost:', self port, '/test/auth'.
	self assert: resp code = 401.

	user := 'user'. password := 'pass'.
	resp := WebClient httpGet: 'http://localhost:', self port, '/test/auth'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (reqHeader beginsWith: 'Digest ').
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 3/27/2010 01:46'!
testFields
	"Test client and server handling simple fields"

	| resp |
	server addService: '/fields' action:[:req | 
		req send200Response: (String streamContents:[:s|
			req fields keys asArray sort do:[:key | s cr; nextPutAll: key,'=', (req fields at: key)].
		]).
	].

	resp := WebClient httpGet: 'http://localhost::', self port, '/fields?foo=123&bar=yoho'.
	self assert: resp code = 200.
	self assert: resp content = '
bar=yoho
foo=123'.

	resp := WebClient httpPost: 'http://localhost::', self port, '/fields' 
				content: 'foo=123&bar=yoho'
				type: 'application/x-www-form-urlencoded'.
	self assert: resp code = 200.
	self assert: resp content = '
bar=yoho
foo=123'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 4/1/2010 18:28'!
testInvalidCookies
	"Test client and server handling of cookies"

	| resp client |
	"Sends the testcookie back if it's set"
	server addService: '/' action:[:req | 
		req send200Response: (req cookieAt: 'testcookie').
	].

	"Sets the cookie"
	server addService: '/setcookie' action:[:req | | domain path expires secure |
		domain := req fields at: 'domain' ifAbsent:[nil].
		path := req fields at: 'path' ifAbsent:[nil].
		expires := req fields at: 'expires' ifAbsent:[nil].
		secure := req fields at: 'secure' ifAbsent:['false'].
		req send200Response: 'ok' contentType: 'text/plain'  do:[:reply| 
			reply setCookie: 'testcookie' value: '123' path: path 
				expires: expires domain: domain secure: secure = 'true'.
		]
	].

	client := WebClient new.
	resp := client httpGet: 'http://localhost:', self port, '/setcookie?path=/'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (resp setCookieAt: 'testcookie') value = '123'.
	resp := client httpGet: 'http://localhost:', self port, '/'.
	self assert: resp code = 200.
	self assert: resp content = '123'.

	client := WebClient new.
	resp := client httpGet: 'http://localhost:', self port, '/setcookie?domain=.foo.com'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (resp setCookieAt: 'testcookie') value = '123'.

	resp := client httpGet: 'http://localhost:', self port, '/'.
	self assert: resp code = 200.
	self deny: resp content = '123'.

	client := WebClient new.
	resp := client httpGet: 'http://localhost:', self port, '/setcookie?domain=.com'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (resp setCookieAt: 'testcookie') value = '123'.
	resp := client httpGet: 'http://localhost:', self port, '/'.
	self assert: resp code = 200.
	self deny: resp content = '123'.

	client := WebClient new.
	resp := client httpGet: 'http://localhost:', self port, '/setcookie?secure=true'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
	self assert: (resp setCookieAt: 'testcookie') value = '123'.
	resp := client httpGet: 'http://localhost:', self port, '/'.
	self assert: resp code = 200.
	self deny: resp content = '123'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 23:22'!
testListenOnInterface
	"Test that listening on a particular interface listens only on that interface"

	| localHostAddr resp localHostName client |
	server destroy.		"kill old server"

	localHostAddr := NetNameResolver localHostAddress.
	localHostAddr = #(127 0 0 1) asByteArray ifTrue:[^self]. "skip test"

	localHostName := String streamContents:[:s|
		localHostAddr do:[:b| s print: b] separatedBy:[s nextPut: $.].
	].

	server := WebServer new listenOn: self port interface: localHostAddr.
	server addService: '/' action:[:req| req send200Response: 'ok'].

	client := WebClient new.
	client timeout: 1.
	self should:[resp := client httpGet: 'http://localhost::', self port, '/test'] raise: Error.

	resp := client httpGet: 'http://', localHostName, ':', self port, '/test'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 19:36'!
testNestedAction
	"Test handling of nested actions"

	| resp |
	server addService: '/test' action:[:req | req send200Response: 'ok'].
	server addService: '/test/42' action:[:req | req send200Response: '42'].

	resp := WebClient httpGet: 'http://localhost::', self port, '/test'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.

	resp := WebClient httpGet: 'http://localhost::', self port, '/test/42'.
	self assert: resp code = 200.
	self assert: resp content = '42'.

	resp := WebClient httpGet: 'http://localhost::', self port, '/test/43'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 4/10/2010 11:45'!
testPersistentAuthRedirectSession
	"Run a test for a full auth-redirect-cookie loop.
	Ensure that the connection is persistent for the entire loop."

	| loginOK finalUrl finalFields firstRedirect client resp url |
	server addService: '/login' action:[:req | 
		server authenticate: req realm: 'test' methods: #(digest) do:[ | id |
			loginOK := true.
			server sessionAt: (id := UUID new hex) put: ''.
			req send302Response: (req fields at: 'url' ifAbsent:['/']) unescapePercents
				do:[:reply| reply setCookie: 'session' value: id path: '/']]].
	server addService: '/action' action:[:req | 
		(server sessionAt: (req cookieAt: 'session')) ifNil:[
			firstRedirect := true.
			req send302Response: '/login?url=', req rawUrl encodeForHTTP.
		] ifNotNil:[
			finalUrl := req url.
			finalFields := req fields.
			req send200Response: 'ok'.
		].
	].
	firstRedirect := loginOK := false.
	server passwordAt: 'squeak' realm: 'test' put: 'foo'.

	client := WebClient new.
	client allowRedirect: false.
	client authCredentialsBlock:[:cl :realm :fail|
		self assert: client == cl.
		self assert: client isConnected.
		client username 
			ifNil:[client username: 'squeak'; password: 'foo']
			ifNotNil:[fail value]].
	url :=  'http://localhost:', self port, '/action/foo/bar?string=hello&number=42'.
	[resp := client httpGet: url.
	resp code = 302] whileTrue:[
		self assert: client isConnected.
		url := resp headerAt: 'Location'.
	].
	self assert: client isConnected.
	self assert: resp code = 200.
	self assert: firstRedirect.
	self assert: loginOK.
	self assert: finalUrl = '/action/foo/bar'.
	self assert: finalFields size = 2.
	self assert: (finalFields at: 'string') = 'hello'.
	self assert: (finalFields at: 'number') = '42'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 4/1/2010 18:10'!
testRedirect
	"Test client and server handling of redirects"

	| resp client |
	server addService: '/foo' action:[:req | 
		req send302Response: '/bar'
	].
	server addService: '/bar' action:[:req | 
		req send200Response: 'redirect ok'
	].

	client := WebClient new.
	resp := client httpGet: 'http://localhost::', self port, '/foo'.
	self assert: resp code = 200.
	self assert: resp content = 'redirect ok'.

	client allowRedirect: false.
	resp := client httpGet: 'http://localhost::', self port, '/foo'.
	self assert: resp code = 302.

	client allowRedirect: true.
	resp := client httpGet: 'http://localhost::', self port, '/foo'.
	self assert: resp code = 200.
	self assert: resp content = 'redirect ok'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 19:35'!
testRedirectLoop
	"Test client handling of redirect loops"

	| resp |
	server addService: '/foo' action:[:req | 
		req send302Response: '/bar'
	].
	server addService: '/bar' action:[:req | 
		req send302Response: '/foo'
	].

	resp := WebClient httpGet: 'http://localhost::', self port, '/foo'.
	self assert: resp code = 302.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 4/12/2010 15:02'!
testRedirectTrailingSlash
	"Special test to ensure that a redirect from /foo to /foo/ works"

	| resp client |
	server addService: '/foo' action:[:req | 
		req send302Response: '/foo/'
	].
	server addService: '/foo/' action:[:req | 
		req send200Response: 'redirect ok'
	].

	client := WebClient new.
	resp := client httpGet: 'http://localhost::', self port, '/foo'.
	self assert: resp code = 200.
	self assert: resp content = 'redirect ok'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 19:35'!
testServerError
	"Test server handling of errors"

	| resp |
	server addService: '/test' action:[:req | self error: 'boom'].

	resp := WebClient httpGet: 'http://localhost::', self port, '/test'.
	self assert: resp code = 500.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 2/25/2010 19:34'!
testSimpleServerAction
	"Test client and server handling simple defaults"

	| resp |
	server addService: '/test' action:[:req | req send200Response: 'ok'].

	resp := WebClient httpGet: 'http://localhost::', self port, '/test'.
	self assert: resp code = 200.
	self assert: resp content = 'ok'.
! !

!WebClientServerTest methodsFor: 'tests' stamp: 'ar 5/4/2010 15:44'!
testStreaming
	"Run a test for a full auth-redirect-cookie loop"

	| resp amount stream |
	amount := 128*1024.
	server addService: '/streaming' action:[:req|
		req stream200Response: (String new: amount) readStream size: amount.
	].
	resp := WebClient httpGet: 'http://localhost:', self port, '/streaming'.
	self assert: resp isSuccess.
	stream := WriteStream on: String new.
	resp streamTo: stream size: resp contentLength progress: nil.
	self assert: stream position = amount.
! !
